/* Copyright (C) Bill Buckels 2008 */

#ifdef CMAIN

/* if you are going to poke around
   this is the best way that I have found */

char *poke = (char *)0;  /* a zero pointer */
char *peek = (char *)0;


/* memory map for creating C64 graphics programs in Aztec C

   0x800 - (2048) - basic load address
   0x810 - binary base address
   0x4000 - data load address - advance as required
   0x4400 - code load address - advance as required

   for embedding a HIRES image
   insert vram before bitmap

   1000 bytes for vram storage
   8000 bytes for bitmap

   insert bitmap at default screen address when program loads
   insert vram 1K before bitmap
   0x2000
   0x1c00

   the bitmap is immediately displayed on the screen and vram
   storage is then moved immediately to the actual vram address

   for embedding a 1024 byte 8 x 8 fontset (characters 0-127)
   insert fontset 2K before bitmap
   0x1800

   I have found in practice that no insertion of data can be done
   below these areas.


*/

/* I am embedding a 1024 byte fontset below the vstore area.
   this area of memory can be overwritten and used as a static buffer */
char *ramfont = (char *)6144;
char *xfont = (char *)6144;
char *dirptr = (char *)6144;
int dirtop = 6144;

/* I am embedding a HIRES vram color map 1024 bytes below the screen area.
   After the initial load of the program this is no longer required
   and this  area of memory can be overwritten and used as a static buffer */

char *vstore = (char *)7168;

/* between the two areas of memory above you have up to 2048 bytes of static
   memory preloaded with data that may not be required after the program loads
   and if so can potentially be used as a data store without the need for
   dynamic memory allocation. */

char *bmp = (char *)8192;
char *vram = (char *)1024;
char *cram = (char *)55296;

char *memsetup = (char *)53272;
char *scrsetup = (char *)53265;

/* for multicolor mode */
char *multimode = (char *)53270;

/* border color at 53280 is immediately followed by background color at 53281 */
char *bordercolor = (char *)53280;

/* text screen pointers */
char *txtaddr = (char *)1024;
char *coladdr = (char *)55296;

/* keyboard pointers */
char *kpress = (char *)203;
char *kbuf =  (char *)631;
char *kbuflen = (char *)198;

/* text cursor pointers */
char *currow = (char *)214;
char *curcol = (char *)211;

#else

/* general purpose */
extern char *poke;
extern char *peek;

/* graphics */
extern char *ramfont;
extern char *xfont;
extern char *dirptr;
extern int dirtop;

extern char *vstore;
extern char *bmp;
extern char *vram;
extern char *cram;
extern char *memsetup;
extern char *scrsetup;
extern char *multimode;
extern char *bordercolor;

/* text screen pointers */
extern char *txtaddr;
extern char *coladdr;

/* keyboard pointers */
extern char *kpress;
extern char *kbuf;
extern char *kbuflen;

/* text cursor pointers */
extern char *currow;
extern char *curcol;

#endif

