/* c64chr.h (C) Copyright Bill Buckels 2007 */


/* Note: You probably shouldn't use headers as includes in your C64 programs */
/*       unless absolutely required. */

/*       Consider using them for information only unless building library functions */
/*       and even then use them sparingly. */

/*       Isolate the values you need and */
/*       put either the defines or their constants directly into your programs. */

/*       The issue here is that the compiler runs out of memory if too many */
/*       defines are used, because it needs to store a list of substitutions */
/*       and the string literals take up memory quite quickly. */

/*       keeping the length of a define short is also a good practice to follow */
/*       for the same reason and so probably is keeping variable names short */

/*       If you use functions that are not of the default type of int */
/*       use the old k&r terse prototype directly in your program */



/*

ASCII AND CHR$ CODES


  This  appendix  shows  you  what  characters  will  appear if you PRINT
CHR$(X),  for  all  possible  values  of  X. It will also show the values
obtained by typing PRINT ASC("x"), where x is any character you can type.
This  is  useful in evaluating the character received in a GET statement,
converting upper/lower case,  and printing character based commands (like
switch to upper/lower case) that could not be enclosed in quotes.

+-----------------+-----------------+-----------------+-----------------+
|  PRINTS   CHR$  |  PRINTS   CHR$  |  PRINTS   CHR$  |  PRINTS   CHR$  |
+-----------------+-----------------+-----------------+-----------------+
|             0   |    0       48   |            96   | {black}   144   |
|             1   |    1       49   |            97   |   {up}    145   |
|             2   |    2       50   |            98   | {rvs off} 146   |
|             3   |    3       51   |            99   | {clear}   147   |
|             4   |    4       52   |           100   |  {inst}   148   |
| {white}     5   |    5       53   |           101   | {brown}   149   |
|             6   |    6       54   |           102   | {lt. red} 150   |
|             7   |    7       55   |           103   | {grey 1}  151   |
| disSHIFT+C= 8   |    8       56   |           104   | {grey 2}  152   |
| enaSHIFT+C= 9   |    9       57   |           105   | {lt.green}153   |
|            10   |    :       58   |           106   | {lt.blue} 154   |
|            11   |    ;       59   |           107   | {grey 3}  155   |
|            12   |    <       60   |           108   | {purple}  156   |
| return     13   |    =       61   |           109   | {left}    157   |
| lower case 14   |    >       62   |           110   | {yellow}  158   |
|            15   |    ?       63   |           111   |  {cyan}   159   |
|            16   |    @       64   |           112   |  SPACE    160   |
|  {down}    17   |    A       65   |           113   |           161   |
| {rvs on}   18   |    B       66   |           114   |           162   |
|  {home}    19   |    C       67   |           115   |           163   |
|  {del}     20   |    D       68   |           116   |           164   |
|            21   |    E       69   |           117   |           165   |
|            22   |    F       70   |           118   |           166   |
|            23   |    G       71   |           119   |           167   |
|            24   |    H       72   |           120   |           168   |
|            25   |    I       73   |           121   |           169   |
|            26   |    J       74   |           122   |           170   |
|            27   |    K       75   |           123   |           171   |
|  {red}     28   |    L       76   |           124   |           172   |
| {right}    29   |    M       77   |           125   |           173   |
| {green}    30   |    N       78   |           126   |           174   |
|  {blue}    31   |    O       79   |           127   |           175   |
|  SPACE     32   |    P       80   |           128   |           176   |
|    !       33   |    Q       81   | {orange}  129   |           177   |
|    "       34   |    R       82   |           130   |           178   |
|    #       35   |    S       83   |           131   |           179   |
|    $       36   |    T       84   |           132   |           180   |
|    %       37   |    U       85   |    f1     133   |           181   |
|    &       38   |    V       86   |    f3     134   |           182   |
|    '       39   |    W       87   |    f5     135   |           183   |
|    (       40   |    X       88   |    f7     136   |           184   |
|    )       41   |    Y       89   |    f2     137   |           185   |
|    *       42   |    Z       90   |    f4     138   |           186   |
|    +       43   |    [       91   |    f6     139   |           187   |
|    ,       44   |  pound     92   |    f8     140   |           188   |
|    -       45   |    ]       93   |shift+ret. 141   |           189   |
|    .       46   |    ^       94   |upper case 142   |           190   |
|    /       47   |{arrow left}95   |           143   |           191   |
+-----------------+-----------------+-----------------+-----------------+

CODES 192-223 SAME AS  96-127
CODES 224-254 SAME AS 160-190
CODE 255 SAME AS 126

*/

#define ENTER_KEY     13

#define UP_ARROW      145
#define DOWN_ARROW	  17
#define LEFT_ARROW	  157
#define RIGHT_ARROW	  29

#define F1 133
#define F3 134
#define F5 135
#define F7 136
#define F2 137
#define F4 138
#define F6 139
#define F8 140

#define CLEAR_HOME 147

#define REV_ON  18
#define REV_OFF 146

#define UPPER_TEXT  142
#define LOWER_TEXT  14

#define BLACK_TEXT   144
#define BWHITE_TEXT	   5
#define RED_TEXT	  28
#define CYAN_TEXT    159
#define PURPLE_TEXT  156
#define GREEN_TEXT	  30
#define BLUE_TEXT	  31
#define YELLOW_TEXT  158

#define ORANGE_TEXT  129
#define BROWN_TEXT   149
#define LRED_TEXT    150
#define DGRAY_TEXT   151
#define MGRAY_TEXT   152
#define LGREEN_TEXT  153
#define LBLUE_TEXT   154
#define LGRAY_TEXT   155
