/* c64poke.h (C) Copyright Bill Buckels 2007 */

/* Note: You probably shouldn't use headers as includes in your C64 programs */
/*       unless absolutely required. */

/*       Consider using them for information only unless building library functions */
/*       and even then use them sparingly. */

/*       Isolate the values you need and */
/*       put either the defines or their constants directly into your programs. */

/*       The issue here is that the compiler runs out of memory if too many */
/*       defines are used, because it needs to store a list of substitutions */
/*       and the string literals take up memory quite quickly. */

/*       keeping the length of a define short is also a good practice to follow */
/*       for the same reason and so probably is keeping variable names short */

/*       If you use functions that are not of the default type of int */
/*       use the old k&r terse prototype directly in your program */


/* poke addresses */


#define POKE_BORDER 53280
#define POKE_BACKGROUND 53281
#define POKE_CHARSET 53272

/* poke values for character sets */
#define UPPER_CASE 21
#define MIXED_CASE 23


/* poke values for colors */
#define C64_BLACK 0
#define C64_BWHITE 1
#define C64_RED	2
#define C64_CYAN 3
#define C64_PURPLE 4
#define C64_GREEN 5
#define C64_BLUE 6
#define C64_YELLOW 7
#define C64_ORANGE 8
#define C64_BROWN 9
#define C64_LRED 10
#define C64_DGRAY 11
#define C64_MGRAY 12
#define C64_LGREEN 13
#define C64_LBLUE 14
#define C64_LGRAY 15


/*

The  64's  text screen can hold 1000 characters (40 columns by 25 lines),
so there  are  1000  memory  locations set aside to represent what is on
the screen.

The 64's text screen memory normally begins at memory location 1024 and
ends at  location 2023.  Location 1024 is the upper left corner of the
screen.


The color memory map begins at location  55296  (upper  left  corner),
and  continues  on  for 1000 locations. You use the same color codes, 0
through 15,  that you used to change border and background colors, to
directly change character color.


*/
