/* cbinlode.c (C) Copyright Bill Buckels 2008 */


/* a picture viewer for the Commodore 64   */
/* written in Manx Aztec C65 Version 1.05h */
/* runs under Commodore BASIC 2            */
/* displays BSaved HIRES and Multi-Color Images */
/* requires a text script called piclist   */



/*

Image Support

The images supported by this program are either split images of the type
created by my ClipShop program and require the file extensions that ClipShop
provides, or are Art Studio, Doodle Draw or Koala Paint uncompressed images,
with the file extensions of .ART, .DD, or .KOA that are supported by
ClipShop and other C64 image conversion utilities currently in use.

Images are read directly from disk into to their respective memory areas.

The images created by ClipShop store the palette(s) in a separate file from
the bitmap. Bitmaps are all 8002 bytes long and palette files are all 1002
bytes long. The first two bytes are a BSaved Header followed by data.

In the case of ClipShop's multicolor images, a third file (a second palette
file) contains the third color and background color. The background is in
the high nibble and is identical in all of the 1000 positions used by C64
palettes.

The first byte of the second palette is used to set the border (low nibble)
and background (high nibble) colors and then transferred to the first
multicolor position in memory, followed by reading the other 999 bytes
containing the rest of the third colors in the second palette directly to
the remaining 999 multicolor positions in memory.

In all cases, each ClipShop file is read in 2 passes, with the first pass
removing the header information and the second pass being read directly to
memory. Since I am doing a raw read, there probably isn't a more efficient
way to do this.

In the case of the ART, DD, and KOA images I do almost the same thing but
all bitmap and palette information comes from a single file.

Image Load Strategy

The image file extension drives the loader, but if a file name without an
extension is entered is typed-in I use the following strategy:

First I try to open a HIRES BSaved file with a single palette. I depend
entirely on naming convention so I expect an extension of ".BHI" for the
bitmap file paired with a palette file of ".VHI".

Second, if that fails, I try to open a Multi-Color BSaved File with an
extension of ".BMC" and 2 palette files with extensions of ".VMC" and ".CMC"
respectively again depending entirely on naming convention.

If both of those strategies fail, then I try for a .KOA, .ART, and .DD file
respectively.

If that fails, I just give-up and assume that whatever they are loading is a
HIRES BSaved image without a palette, so I set a black and white palette
manually and let them use whatever naming they like.

This part can be expanded to test for other image types that the C64
supports. The header and file information could have been used in the loader
to make it more robust in auto-detecting the many C64 graphics that exist,
as well as file length and load addresses stored in the image. It is only in
the interests of keeping this simple that I am using file extensions as "the
easy way out" in my loader routine and limiting my strategy to keep this
program smaller and more readable.

*/



/* memory map for creating C64 graphics programs in Aztec C

0x800 - (2048) - basic load address
0x810 - binary base address
0x4000 - data load address (advance as necessary if problems)
0x4400 - code load address (advance as necessary for more data)

for embedding a HIRES image
insert vram before bitmap

1000 bytes for vram storage
8000 bytes for bitmap

insert bitmap at default screen address when program loads
insert vram 1K before bitmap
0x2000
0x1c00

the bitmap is immediately displayed on the screen and vram
storage is then moved immediately to the actual vram address

for embedding a 1024 byte 8 x 8 fontset (characters 0-127)
insert fontset 2K before bitmap
0x1800

I have found in practice that no insertion of data can be done
below these areas.


*/
#define CMAIN 1

#include <poke.h>
#include <colors.h>
#include <stdio.h>

#define NONE 0
#define BHI 1
#define BMC 2
#define ART 3
#define DD  4
#define KOA 5

int itype = NONE;

char tmpbuf[3];

char image[40];
char *palette[40];
char *cmap[40];

/* since less than 25 C64 images can be placed on a CBINLODE disk
   the following pointer array is sufficient for memory
   storage of image names from the PICLIST.
   This allows the PICLIST to be opened once at the start of the
   program then to be read into memory referenced by this array */
char *inames[25];


nocr(ptr, cr)
char *ptr;
char cr;
{
	int idx;

	if (cr == '.')itype = NONE;

	for (idx=0;ptr[idx] != 0; idx++) {
	    if (ptr[idx] == cr) {
		   ptr[idx] = 0;

		   if (cr == '.') {
			  if (ptr[idx+1] == 'B' && ptr[idx+2] == 'H' && ptr[idx+3] == 'I') {
				  itype = BHI;
				  break;
			  }
			  if (ptr[idx+1] == 'B' && ptr[idx+2] == 'M' && ptr[idx+3] == 'C') {
				  itype = BMC;
				  break;
			  }
			  if (ptr[idx+1] == 'A' && ptr[idx+2] == 'R' && ptr[idx+3] == 'T') {
				  itype = ART;
				  break;
			  }
			  if (ptr[idx+1] == 'D' && ptr[idx+2] == 'D') {
				  itype = DD;
				  break;
			  }
			  if (ptr[idx+1] == 'K' && ptr[idx+2] == 'O' && ptr[idx+3] == 'A') {
				  itype = KOA;
				  break;
			  }
		   }

		   break;
		}

	}

}


FILE *fp = NULL;
char buffer[128];

int xkey = 90; /* exit = Z */

main()
{
	int c,fh,idx,jdx;

    scr_clear();

    /* background color, bordercolor */
    scr_back(C64_LBLUE,C64_BLUE);

	/* set to hires mode 320 x 200 and display the logo */
    setcrtmode(1);
    setlogo();

    pfont("Please Wait...",9,13,0,1,-1);

    /* open the piclist and read the images into the buffer */
    /* I am using a fixed field length of 40 characters in the buffer */
    for (idx = 0;idx < 25; idx++)inames[idx] = NULL;
    fp = fopen("PICLIST","r");
	if (fp!=NULL) {
		  idx = 0;
		  jdx = 0;
		  while (fgets(buffer,128,fp) != NULL) {
			  if (idx > 24)break;
	          nocr(buffer,'\n');
	          buffer[39]=0;
	          inames[idx] = (char *)&vstore[jdx];
	          strcpy(inames[idx],buffer);
	          idx++;
	          jdx +=40; /* 40 character field length */

		  }
		  fclose(fp);

    }


    /* clear all keypressess */
    while (kbhit()); /* wait for key release */
    kbflush();

    pfont("Press Any Key.",9,13,0,1,-1); /* erase please wait */

    while (!kbhit()); /* wait for key press */
    /* clear last palette */
    gclr(-1,0,0);
    setcrtmode(0);
    scr_clear();

    /* clear bmp */
    gclr(0,-1,-1);
    while (kbhit()); /* wait for key release */
    /* clear the buffer */
    kbflush();

BEGERR:


    /* home cursor, clear screen */
    scr_clear();

    /* background color, bordercolor */
    scr_back(C64_LBLUE,C64_BLUE);

    scr_txtcolor(C64_BWHITE); /* bright white text */

    printf("CBINLOAD.PRG BY BILL BUCKELS 2007\n\nPICTURES:\n\n");

	jdx = 65; /* ascii 'A' */
	for (idx = 0; idx < 25; idx++) {
	   if (inames[idx] == NULL)break;
	   printf("  %c - %s\n",jdx, inames[idx]);
	   jdx++;
	}
	printf("  %c - QUIT TO BASIC\n",xkey);


RENTER:

    printf("\nENTER PICTURE:");
    gets(buffer);
    if (buffer[0] == 0 || (buffer[1] == 0 && buffer[0] == xkey)) goto ENDERR;

    jdx=0;
    if (buffer[1] == 0) {
	   jdx = 999;
	   if (buffer[0] > 64 && buffer[0] < 90) {
	        idx = buffer[0] - 65;
	        if (inames[idx] != NULL){
				strcpy(buffer,inames[idx]);
				jdx = 0;
			}
	   }

	}
	if (jdx == 999) {
		printf("INVALID MENU OPTION.");
		goto RENTER;
	}


    strcpy(image,buffer);
    nocr(image,'.');
    strcpy(palette,image);
    strcpy(cmap,image);


    switch(itype) {
	   case KOA:
	   case ART:
	   case DD:
	   case BMC:
	   case BHI:
	        fh = open(buffer,0); break;
	   default:
	        /* try to open as hires with palette */
	        strcat(image,".BHI");
	        fh = open(image,0);
	        if (fh!= -1) {
				itype = BHI; break;
			}
	        /* if that fails try to open as multicolor with palette and cmap */
	        strcpy(image, palette);
			strcat(image,".BMC");
	        fh = open(image,0);
	        if (fh!= -1) {
				itype = BMC; break;
			}
			/* if that fails try to open as Koala Painter */
			strcpy(image, palette);
			strcat(image,".KOA");
	        fh = open(image,0);
	        if (fh!= -1) {
				itype = KOA; break;
			}
			/* if that fails try to open as Art */
			strcpy(image, palette);
			strcat(image,".ART");
	        fh = open(image,0);
	        if (fh!= -1) {
				itype = ART; break;
			}
			/* if that fails try to open as Doodle Draw */
			strcpy(image, palette);
			strcat(image,".DD");
	        fh = open(image,0);
	        if (fh!= -1) {
				itype = DD; break;
			}
			/* if all fail assume hires without palette */
			fh = open(buffer,0); break;

	}

    if (fh == -1) {
		printf("CAN'T OPEN %s",buffer);
		goto RENTER;
	}


	scr_clear();

	setcrtmode(1);

    switch(itype) {
		case KOA:
			/* Koala Painter */
			/* second step required for multicolor graphics mode */
			setcrtmode(2);

		    palette[0] = 0;
		    cmap[0] = 0;

		    break;

	    case ART:
		    palette[0] = 0;
		    cmap[0] = 0;
		    break;

	    case DD:
	        /* doodle draw stores the palette before the bitmap */
	        palette[0] = 0;
	        cmap[0] = 0;
	        read(fh,tmpbuf,2);
	        read(fh,vram,1000);
	        read(fh,buffer,22); /* padding - the last 2 bytes will be thrown away below */
	        break;

		case BMC:
		    /* second step required for multicolor graphics mode */
            setcrtmode(2);

            strcat(palette,".VMC");
            strcat(cmap,".CMC");

            break;

		case BHI:
		default:
		    strcat(palette,".VHI");
	}

    read(fh,tmpbuf,2);
    read(fh,bmp,8000);
    if (itype == KOA || itype== ART)read(fh,vram,1000);
    if (itype == KOA) {
		read(fh,cram,1000);
		read(fh,buffer,1);
		/* background color, bordercolor */
        scr_back((buffer[0] & 0xf),(cram[0] & 0xf));
	}

    close(fh);

    if (palette[0] != 0) {
    	fh = open(palette,0);
    	if (fh != -1) {
			read(fh,tmpbuf,2);
			read(fh,vram,1000);
			close(fh);
		}
		else {
			for (idx = 0; idx < 1000; idx++)vram[idx] = C64_BWHITE;

		}
	}

    /* multicolor mode */
    if (itype == BMC) {
            fh = open(cmap,0);
			if (fh != -1) {
				read(fh,tmpbuf,3);
				/* background color, bordercolor */
                scr_back((tmpbuf[2] >> 4),(tmpbuf[2] & 0xf));
                cram[0] = tmpbuf[2];
				read(fh,&cram[1],999);
				close(fh);
	        }
	}


    /* clear all keypressess */
    while (kbhit());
    kbflush();
    c = getch();

    /* clear last palette and bitmap */
    gclr(0,0,0);

    setcrtmode(0);

    if (c!=xkey) goto BEGERR;

ENDERR:

	scr_reset();
	exit(0);
}