/* songdir.c (C) Copyright Bill Buckels 2008 */

/* This program is an original Aztec C65 SID chip demonstration
program. It provides several preset instruments with additional
settings and loads sound files from disk.

It has been built under Windows XP in the Aztec C65
cross-development environment for the C64.

This program links with my B64NAT.LIB and the standard
native mode C64NAT.LIB provided by Manx.

Like any C64 native mode program, this program is loaded from BASIC.

This program uses the 3 voice SID chip that the C64 sported. This
was a fairly complex bit of hardware and you should probably study
the Commodore 64 Programmer's Reference Guide if you require more
clarification. You should also probably understand some of the
physics of sound and perhaps some musical theory, and maybe even
some electrical engineering to really grasp what is going on here.

None of that is a pre-requisite to having some fun with this of
course.

As far as the sound files go they are a series of 2 byte
note,duration pairs that I have converted from olde MS-DOS bit
music of frequency, duration pairs. The note values are given
using the general midi note, but I have stayed with the orginal
ibm duration of 18.2 ticks per second.

The program screen was created in the IBM-PC program TheDraw then
converted to C64 format.

The utilities to do all this conversion are in the TOOLS
directory in this environment complete with source code and
can be referred to for more info.

This is a relatively non-trivial program, and some of the other
samples should be worked through first unless you know C pretty
well and have mucked-with MS-DOS and Commodore 64's in one of
your past-lives. OTOH there are lots of people out there who
are smarter than I who will probably take one look at this
and understand what I am doing and just shake their heads and
wonder why.

For the rest of us, there are manuals and samples, and mastercard.

'Nuff Said.

*/

#define CMAIN 1

#include <poke.h>
#include <colors.h>
#include <sid.h>

/* constants must be short and kept to a minimum */
#define TRUE 1
#define FALSE 0
#define FOFF 17


/* navigation echo to screen */
/* the following coordinates are based on SCR.BSV
   which is the source file for the BLOADed screen */

/* the vars are the same format for each of the
   4 areas of the screen which control song play...

   they consist of a starting row, col,
   the new index, the old index, and the length of the
   menu bar.
*/

/* songs */
int srow = 12, scol = 5, slen = 16;
int sidx = 0;
int sodx = 0;

/* instruments */
int irow = 4, icol = 26, ilen = 13;
int iidx = 0;
int iodx = 0;

/* tempo */
int trow = 23, tcol = 26, tlen = 6;
int tidx = 0;
int todx = 0;

/* voice settings */
int vrow = 4, vcol = 10, vlen = 10;
int vidx = 0;
int vodx = 0;

/* harmony settings */
int hrow = 8, hcol = 10, hlen = 10;
int hidx = 0;
int hodx = 0;

char buffer[40];
char cursong[FOFF];
char songs[10][FOFF];

main()
{
   int idx, fileidx;
   int ch, playkey, curidx;

   int voice = 1; /* default to select voice 1 */
   int voice1 = 0;
   int voice2 = -1;
   int voice3 = -1;
   int tempo = 812;

   scr_mixed(); /* put the C64 in mixed case mode */
   scr_clear(); /* clear screen */

   /* set background to black */
   /* set border to green */
   scr_back(C64_BLACK,C64_GREEN);

   fileidx = dlist("SONGDIR.PRG","SIB");
   if (fileidx == 0) {
	   printf("nO sib fILES fOUND!\n");
       /* clear all keypressess */
       while (kbhit());
       /* clear the buffer */
       kbflush();
	   goto BOTTOM;
   }

   /* load a binary screen - use default load addresses */
   /* second arg is set to 0 - otherwise override addr */
   bload("SCR.BXC",0);
   bload("SCR.BXD",0);

   /* set-up the defaults */
   /* the displays reflect the initial presets
      of a single voice piano */

   /* use default color already in the BLoaded Screen */
   curidx = 0;
   cursong[0] = 0;
   /* the first 10 songs are loaded into the
      display... they can press spacebar for
      the next 10 */
   for (idx = 0; idx < 10; idx++) {
	   if (curidx == fileidx)curidx = 0;
	   strcpy(songs[idx],&dirptr[curidx*FOFF]); /* file */
	   sprintf(buffer,"%-16s",songs[idx]); /* screen */
       outtext(buffer,srow+idx,scol,-1,0);
       curidx++;
   }
   /* reverse initial song */
   revtext(srow+sidx,scol,slen,1);

   /* reverse initial instrument */
   revtext(irow+iidx,icol,ilen,1);

   /* reverse initial voice setting */
   outtext("A",vrow,vcol+8,-1,0);
   revtext(vrow+vidx,vcol,vlen,1);

   /* reverse initial harmony settings */
   revtext(hrow+hidx,hcol,hlen,1);

   /* reverse initial tempo setting */
   revtext(trow+tidx,tcol,tlen,1);

   /* clear all keypressess */
   while (kbhit());
   /* clear the buffer */
   kbflush();

/* the code below is mostly cosmetic p**p to enhance the
user experience. having said that, communicating the
interface of these jukebox programs has always been a chore
so s*ck-it-up little bear:) this is the way it was done
in MS-DOS and on the Apple II back in those days as well.

No worries tho'. My email works if you get really lost.
*/

TOP:;

    playkey = FALSE;

    ch = toupper(getch());

    if (ch == 'X')goto BOTTOM;

    /* hotkeys for instrument selection - A-R */
    if (ch > 64 && ch < 83) {
        iidx = ch-65;
        revtext(irow+iodx,icol,ilen,0);
        iodx = iidx;
        buffer[0] = ch;
        buffer[1] = 0;
        outtext(buffer,vrow+vidx,vcol+8,-1,1);
        switch(voice) {
		  case 2:  voice2 = ch-65; break;
		  case 3:  voice3 = ch-65; break;
		  case 1:  voice1 = ch-65; break;
		  default: break;
	    }
	    revtext(irow+iodx,icol,ilen,1);
	}
	/* hotkeys for song selection '1'-'0' */
    /* these keys are for immediate-mode playback */
    /* the arrow key and enter key work as well */
	else if (ch > 47 && ch < 58 ) {
	  if (ch==48) sidx = 9;
	  else sidx = ch-49;
	  revtext(srow+sodx,scol,slen,0);
	  sodx=sidx;
	  revtext(srow+sodx,scol,slen,1);
	  playkey = TRUE;
	}
	/* hotkeys for voice selection and reset */
	/* these are the C64 keycodes for F1-F4 */
	else if (ch == 133 || ch == 137 || ch == 134 || ch == 138) {
		/* reset */
		if (ch==138) {
			vidx = 3;
		    revtext(vrow+vodx,vcol,vlen,0);
	        vodx=vidx;
	        revtext(vrow+vodx,vcol,vlen,1);
			tempo = 812; tidx = 0;
		    revtext(trow+todx,tcol,tlen,0);
	        todx=tidx;
	        revtext(trow+todx,tcol,tlen,1);
			unison(); hidx = 0;
			revtext(hrow+hodx,hcol,hlen,0);
			hodx=hidx;
			revtext(hrow+hodx,hcol,hlen,1);
		    vidx = 0;
		    voice1 = 0;
		    voice2 = -1;
		    voice3 = -1;
		    iidx = 0;
            revtext(irow+iodx,icol,ilen,0);
            iodx = iidx;
            revtext(irow+iodx,icol,ilen,1);
            outtext("A",vrow,vcol+8,-1,0);
            outtext(" ",vrow+1,vcol+8,-1,0);
            outtext(" ",vrow+2,vcol+8,-1,0);
	    }
		if (ch == 133) vidx = 0;
		if (ch == 137) vidx = 1;
		if (ch == 134) vidx = 2;
		voice = vidx+1;
		revtext(vrow+vodx,vcol,vlen,0);
	    vodx=vidx;
	    revtext(vrow+vodx,vcol,vlen,1);
	}
	/* hotkeys for harmony-unison toggle F5-F6 */
	else if (ch == 135 || ch == 139) {
          if (ch == 135) {
			  hidx = 0;
			  unison(); /* unison */
		  }
          else {
			  hidx = 1;
			  harmony(); /* harmony */
		  }
		  revtext(hrow+hodx,hcol,hlen,0);
	      hodx=hidx;
	      revtext(hrow+hodx,hcol,hlen,1);
    }
    /* hot keys for playback speed */
	else if (ch == 'S' || ch == 'T') {
		  /* tempo */
          /* adjust timing loop count for song play */
		  /* this is really the inverse of tempo */
          if (ch == 'S') {
             tidx = 0;
             if (tempo < 1612)tempo += 100;
		  }
          else {
			 tidx = 1;
			 if (tempo > 812)tempo -= 100;
		  }
		  revtext(trow+todx,tcol,tlen,0);
	      todx=tidx;
	      revtext(trow+todx,tcol,tlen,1);
	}
	/* arrow keys for selecting a song visually */
	else if (ch == 145 || ch == 17) {
	      /* up arrow, down arrow */
	      if (ch == 145)sidx -= 1;
	      else sidx += 1;
	      if (sidx > 9)sidx = 0;
	      if (sidx < 0)sidx = 9;
	      /* since arrows are used for selection
	         this is display only...
	         an enter key press or a number key is required to
	         play a song */
		  revtext(srow+sodx,scol,slen,0);
		  sodx=sidx;
		  revtext(srow+sodx,scol,slen,1);

	}
	/* the enter key will play the currently selected song */
	else if (ch == 13) {
		/* enterkey */
		playkey = TRUE;
	}
	/* a spacebar press will page down to the next 10 songs */
	/* and starts over at the beginning when the end is reached */
	else if (ch == 32) {
        /* space bar */
	   for (idx = 0; idx < 10; idx++) {
		   if (curidx == fileidx)curidx = 0;
		   strcpy(songs[idx],&dirptr[curidx*FOFF]); /* file */
		   sprintf(buffer,"%-16s",songs[idx]); /* screen */
		   outtext(buffer,srow+idx,scol,-1,0);
		   curidx++;
	   }
	   /* reverse initial song */
	   revtext(srow+sidx,scol,slen,1);
	}

    /* crux of the biscuit */
    /* play the song */
	if (playkey == TRUE) {
		/* while we are loading, the song goes red */
		/* we are certainly not going to interrupt
		   the file reading process */
		attext(srow+sidx,scol,slen,10);
        attext(vrow+vodx,vcol,vlen,7);
	    attext(trow+todx,tcol,tlen,7);
	    attext(hrow+hodx,hcol,hlen,7);
        attext(irow+iodx,icol,ilen,7);
	    if (songload(sidx) == 0) {
			/* if someone is banging away on a key
			   by the time we are finished loading
			   we just don't play the song */
			if (!kbhit()) {
			   /* the song is white while playing and
			      the other stuff selected in the lists are green */
			   attext(srow+sidx,scol,slen,1);
			   tickvalue(tempo);
			   play(&poke[dirtop],voice1,voice2,voice3);
		    }
		}
	    playkey = FALSE;
		attext(srow+sidx,scol,slen,7);
        attext(vrow+vodx,vcol,vlen,1);
	    attext(trow+todx,tcol,tlen,1);
	    attext(hrow+hodx,hcol,hlen,1);
        attext(irow+iodx,icol,ilen,1);

        /* if they are banging, wait until they finish */
        while (kbhit());
        /* clear the keyboard buffer before returning to the menu */
        kbflush();


	}

    /* any other keys are thrown away */
    goto TOP;


BOTTOM:;


   scr_clear(); /* clear screen */
   scr_upper(); /* reset from mixed case mode */
   /* set background to blue */
   /* set border to lblue */
   scr_back(C64_BLUE,C64_LBLUE);
   /* stuff something in the keyboard to make Aztec C happy */
   kbuf[0] = 'X';
   kbuflen[0]=1;
   getchar();
   scr_clear(); /* clear screen to erase the X */

   exit(0);

}


songload(choice)
{
	/* we are using a static area from the end of the
	   file list to the end of the graphics screen
	   to load the songs. if lots of songs are
	   on the disk and some of them are too large,
	   the large ones will be clipped. */

	int fh;
	int length = 16192 - dirtop;

    /* no need to reload song if already loaded */
	if (strcmp(songs[choice],cursong)== 0)return 0;
	fh = open(songs[choice],0);
	if (fh != -1) {
	     read(fh, &poke[dirtop],length);
	     close(fh);
	     strcpy(cursong,songs[choice]);
	     return 0;
	}
	return 1;
}


