/* songster.c (C) Copyright Bill Buckels 2008 */


/* This program is an original Aztec C65
   SID chip demonstration program. It provides
   several preset instruments with additional
   settings and a small selection of short songs.

   A more polished version of this program which
   reads disk files and has nicer screen controls
   etc, is in the SONDIR sample directory.

   This program is a little easier to read.

   It has been built under Windows XP in the
   Aztec C65 cross-development environment for the C64.

   This program links with my B64NAT.LIB and the standard
   native mode C64NAT.LIB provided by Manx.

   Like any C64 native mode program, this program is
   loaded from BASIC 2.

   This program uses the 3 voice SID chip that the C64
   sported. This was a fairly complex bit of hardware
   and you should probably study the Commodore 64
   Programmer's Reference Guide if you require more
   clarification. You should also probably understand
   some of the physics of sound and perhaps some
   musical theory, and maybe even some electrical
   engineering to really grasp what is going on here.

   None of that is a pre-requisite to having some
   fun with this of course.

   */

#define CMAIN 1

#include <poke.h>
#include <colors.h>
#include <sid.h>

/* musical array created from IBM snd file bingo.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char bingo[]={
255,   1, 60,   4,255,   1, 65,   4,255,   1, 65,   4,255,   1, 60,   4,
255,   1, 60,   4,255,   1, 62,   4,255,   1, 62,   4,255,   1, 60,   4,
255,   1, 60,   4,255,   1, 65,   4,255,   1, 65,   4,255,   1, 67,   4,
255,   1, 67,   4,255,   1, 69,   4,255,   1,255,   4,255,   1, 65,   4,
255,   1,255,   4,255,   1, 69,   4,255,   1,255,   4,255,   1, 69,   4,
255,   1,255,   4,255,   1, 70,   4,255,   1, 70,   4,255,   1, 70,   4,
255,   1,255,   4,255,   1, 67,   4,255,   1,255,   4,255,   1, 67,   4,
255,   1,255,   4,255,   1, 69,   4,255,   1, 69,   4,255,   1, 69,   4,
255,   1,255,   4,255,   1, 65,   4,255,   1,255,   4,255,   1, 65,   4,
255,   1,255,   4,255,   1, 67,   4,255,   1, 67,   4,255,   1, 67,   4,
255,   1, 65,   4,255,   1, 64,   4,255,   1, 60,   4,255,   1, 62,   4,
255,   1, 64,   4,255,   1, 65,   4,255,   1,255,   4,255,   1, 65,   6,
255,   1,0,0};

/* musical array created from IBM snd file bug.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char bug[]={
 50,   1,255,   1, 50,   1,255,   1, 50,   1,255,   1, 55,   1,255,   5,
 59,   1,255,   3, 50,   1,255,   1, 50,   1,255,   1, 50,   1,255,   1,
 55,   1,255,   5, 59,   1,255,   5, 38,   1,255,   1, 38,   1,255,   1,
 43,   1,255,   3, 55,   1,255,   1, 55,   1,255,   1, 54,   1,255,   1,
 54,   1,255,   1, 52,   1,255,   1, 52,   1,255,   1, 50,   8,255,   2,
 50,   1,255,   1, 50,   1,255,   1, 50,   1,255,   1, 54,   1,255,   5,
 57,   1,255,   3, 50,   1,255,   1, 50,   1,255,   1, 50,   1,255,   1,
 54,   1,255,   5, 57,   1,255,   5, 45,   1,255,   1, 45,   1,255,   1,
 38,   1,255,   3, 62,   2, 64,   2, 62,   2, 60,   2, 59,   2, 57,   2,
 55,   8,0,0};

/* musical array created from IBM snd file danube.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char danube[]={
 50,   4, 54,   4, 57,   4, 57,   4,255,   4, 69,   2,255,   2, 69,   2,
255,   6, 66,   2,255,   2, 66,   2,255,   6, 50,   4, 50,   4, 54,   4,
 57,   4, 57,   4,255,   4, 69,   2,255,   2, 69,   2,255,   6, 67,   2,
255,   2, 67,   2,255,   6, 49,   4, 49,   4, 52,   4, 59,   4, 59,   4,
255,   4, 71,   2,255,   2, 71,   2,255,   6, 67,   2,255,   2, 67,   2,
255,   6, 49,   4, 49,   4, 52,   4, 59,   4, 59,   4,255,   4, 71,   2,
255,   2, 71,   2,255,   6, 66,   2,255,   2, 66,   2,255,   6, 50,   4,
 50,   4, 54,   4, 57,   4, 62,   4,255,   4, 74,   2,255,   2, 74,   2,
255,   6, 69,   2,255,   2, 69,   2,255,   6, 50,   4, 50,   4, 54,   4,
 57,   4, 62,   4,255,   4, 74,   2,255,   2, 74,   2,255,   6, 71,   2,
255,   2, 71,   2,255,   6, 52,   4, 52,   4, 55,   4, 59,   2,255,   2,
 59,  14,255,   2, 56,   4, 57,   4, 66,  16, 62,   4, 54,   4, 54,   8,
 52,   4, 59,   8, 57,   4, 50,   4,255,   2, 50,   2, 50,   4,255,   8,
 57,   2,255,   2, 55,   2,255,   6, 57,   2,255,   2, 55,   2,255,   6,
 57,   4, 66,  16, 64,   4, 57,   2,255,   2, 54,   2,255,   6, 57,   2,
255,   2, 54,   2,255,   6, 57,   4, 64,  16, 62,   4, 57,   2,255,   2,
 55,   2,255,   6, 57,   2,255,   2, 55,   2,255,   6, 57,   4, 66,  16,
 64,   4, 57,   4, 62,   4, 64,   4, 66,   4, 69,   8, 67,   4, 66,   2,
 66,   2, 66,   4, 64,   2,255,   2, 62,   4,255,   8,0,0};

/* musical array created from IBM snd file dvorak.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char dvorak[]={
 55,   3,255,   2, 57,   1, 55,   3,255,   2, 57,   1, 59,   3,255,   2,
 62,   1, 64,   3,255,   2, 62,   1, 67,   3,255,   2, 66,   1, 69,   3,
255,   2, 67,   1, 66,   3,255,   2, 69,   1, 67,   3,255,   2, 64,   1,
 62,   3,255,   2, 62,   1, 64,   3,255,   2, 62,   1, 67,   3,255,   2,
 64,   1, 62,   3,255,   2, 59,   1, 57,  24, 55,   3,255,   2, 57,   1,
 55,   3,255,   2, 57,   1, 59,   3,255,   2, 62,   1, 64,   3,255,   2,
 62,   1, 64,   3,255,   2, 66,   1, 69,   3,255,   2, 67,   1, 66,   3,
255,   2, 69,   1, 67,   3,255,   2, 64,   1, 62,   3,255,   2, 62,   1,
 67,   3,255,   2, 55,   1, 57,   6, 62,   6, 55,  18,0,0};

/* musical array created from IBM snd file guonod.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char guonod[]={
 45,   1,255,   2, 38,   1,255,   5, 50,   3, 50,   3, 49,   3, 47,   3,
 49,   3,255,   3, 50,   3, 52,   3,255,   3, 45,   1,255,   2, 38,   1,
255,   5, 50,   3, 50,   3, 49,   3, 47,   3, 49,   3,255,   3, 50,   3,
 52,   3,255,   3, 45,   3, 50,   3,255,   3, 53,   3, 57,   6, 55,   3,
 53,   3,255,   3, 57,   3, 60,   6, 58,   3, 57,   3,255,   3, 61,   3,
 64,   6, 62,   3, 61,   3, 58,   3, 57,   3, 55,   3, 53,   3, 52,   3,
 38,   1,255,   5, 50,   3, 50,   3, 49,   3, 47,   3, 49,   3,255,   3,
 50,   3, 52,   3,255,   3, 45,   1,255,   2, 38,   1,255,   5, 50,   3,
 50,   3, 49,   3, 47,   3, 49,   3,255,   3, 50,   3, 52,   3,255,   3,
 45,   3, 53,   3,255,   3, 57,   3, 60,   6, 58,   3, 57,   3, 55,   3,
 53,   3, 51,   3, 55,   3, 58,   3, 50,   3, 49,   3, 50,   3, 52,   3,
255,   3, 53,   1,255,   2, 52,   9, 50,   1,0,0};

/* musical array created from IBM snd file mexico.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char mexico[]={
 60,   2, 65,   2,255,   2, 60,   2, 65,   2,255,   2, 60,   2, 65,   6,
255,   4, 60,   2, 65,   2, 67,   2, 65,   2, 64,   4, 65,   2, 67,   2,
255,   8, 60,   2, 64,   2,255,   2, 60,   2, 64,   2,255,   2, 60,   2,
 64,   6,255,   4, 60,   2, 64,   2, 65,   2, 64,   2, 62,   4, 64,   2,
 65,   2,255,   6, 72,   2, 71,   2, 72,   2, 69,   2, 68,   2, 69,   2,
 65,   2, 64,   2, 65,   2, 60,   2,255,   4, 57,   2, 58,   2, 60,   2,
 62,   2, 64,   2, 65,   2, 67,   2, 69,   2, 70,   2, 67,   2,255,   4,
 70,   2, 69,   2, 70,   2, 67,   2, 66,   2, 67,   2, 64,   2, 63,   2,
 64,   2, 60,   2,255,   4, 72,   2, 71,   2, 72,   2, 74,   2, 72,   2,
 70,   2, 69,   2, 67,   2, 65,   2,0,0};

/* musical array created from IBM snd file mozart.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char mozart[]={
 63,   2, 62,   2, 62,   4, 63,   2, 62,   2, 62,   4, 63,   2, 62,   2,
 62,   4, 70,   4,255,   4, 70,   2, 69,   2, 67,   4, 67,   2, 65,   2,
 63,   4, 63,   2, 62,   2, 60,   4, 60,   4,255,   4, 62,   2, 60,   2,
 60,   4, 62,   2, 60,   2, 60,   4, 62,   2, 60,   2, 60,   4, 69,   4,
255,   4, 69,   2, 67,   2, 66,   4, 66,   2, 63,   2, 62,   4, 62,   2,
 60,   2, 58,   4, 58,   4,255,   4, 70,   2, 69,   2, 69,   4, 72,   4,
 66,   4, 69,   4, 67,   4, 62,   4,255,   4, 70,   2, 69,   2, 69,   4,
 72,   4, 66,   4, 69,   4, 67,   4, 70,   4, 69,   2, 67,   2, 65,   2,
 63,   2, 62,   4, 54,   4, 55,   4, 57,   4, 58,   4, 60,   2, 58,   2,
 57,   4, 55,   4, 62,   4,255,   4, 73,   8, 74,   2,255,   6, 73,   8,
 74,   2,255,   6, 73,   8, 74,   4, 73,   4, 74,   4, 73,   4, 74,   4,
0,0};


/* musical array created from IBM snd file popwease.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char popwease[]={
 62,   5,255,   1, 62,   2,255,   1, 64,   5,255,   1, 64,   2,255,   1,
 66,   2,255,   1, 69,   2,255,   1, 66,   2,255,   1, 62,   5,255,   1,
 57,   2,255,   1, 62,   5,255,   1, 62,   2,255,   1, 64,   5,255,   1,
 64,   2,255,   1, 66,   8,255,   1, 62,   5,255,   1, 57,   2,255,   1,
 62,   5,255,   1, 62,   2,255,   1, 64,   5,255,   1, 64,   2,255,   1,
 66,   2,255,   1, 69,   2,255,   1, 66,   2,255,   1, 62,   8,255,   1,
 71,   2,255,   1,255,   2,255,   1,255,   2,255,   1, 64,   5,255,   1,
 67,   2,255,   1, 66,   8,255,   1, 62,   8,255,   1,0,0};

/* musical array created from IBM snd file sam.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char sam[]={
 60,   3,255,   1, 60,   3,255,   1, 62,   3,255,   1, 64,   3,255,   1,
 60,   3,255,   1, 64,   3,255,   1, 62,   3,255,   1, 55,   3,255,   1,
 60,   3,255,   1, 60,   3,255,   1, 62,   3,255,   1, 64,   3,255,   1,
 60,   6,255,   1, 55,   6,255,   2, 60,   3,255,   1, 60,   3,255,   1,
 62,   3,255,   1, 64,   3,255,   1, 65,   3,255,   1, 64,   3,255,   1,
 62,   3,255,   1, 60,   3,255,   1, 59,   3,255,   1, 59,   3,255,   1,
 57,   3,255,   1, 59,   3,255,   1, 60,   6,255,   1, 60,   6,255,   3,
0,0};

/* musical array created from IBM snd file stars.snd */
/* array structure is midi note number, duration (18.2 ticks sec) */
char stars[]={
 62,   6, 62,   6, 60,   3, 59,   3, 59,   6, 58,   3, 59,   3, 59,  16,
255,   2, 58,   3, 59,   3, 59,   6, 58,   3, 59,   3, 62,   6, 59,   3,
 62,   3, 60,  12, 57,   6,255,   3, 57,   3, 57,   6, 56,   3, 57,   3,
 57,   6, 56,   3, 57,   3, 60,  16,255,   2, 59,   3, 57,   3, 59,   3,
 62,   9, 64,   9, 64,   3, 57,  16,255,   2, 62,   6, 62,   6, 60,   3,
 59,   3, 59,   6, 58,   3, 59,   3, 59,  16,255,   2, 58,   3, 59,   3,
 59,   6, 58,   3, 59,   3, 60,   3, 59,   3, 57,   5, 54,   1, 57,  12,
 55,   6,255,   3, 55,   3, 55,   6, 54,   3, 55,   3, 58,   6, 57,   3,
 55,   3, 67,  15,255,   3, 55,   3, 57,   3, 59,   3, 62,   1,255,   2,
 55,   3, 57,   3, 59,   3, 62,   1,255,   2, 50,   3, 52,   5, 59,   1,
 57,  12, 55,   1,0,0};


/* instrument list */
/* this list is used on-screen as a menu */
/* the array subscript corresponds with the
   instrument setting subscript. */
char *inlist[] = {
	"A PIANO",
	"B PIANO2",
	"C VIOLIN",
	"D GUITAR",
	"E FLUTE",
	"F FLUTE2",
	"G HARPSICHORD",
	"H HARPSICHORD2",
	"I XYLOPHONE",
	"J XYLOPHONE2",
	"K ORGAN",
	"L ORGAN2",
	"M COLLIAPE",
	"N ACCORDIAN",
	"O ACCORDIAN2",
	"P TRUMPET",
	"Q TRUMPET2",
	"R DRUM",
	"",
	"  S SLOWER",
	"  T FASTER",
    "-"};


char *menu[] = {
    "F1 VOICE 1",
	"F2 VOICE 2",
	"F3 VOICE 3",
	"F4 RESET",
	"F5 UNISON",
	"F6 HARMONY",
	"",
	"1 BINGO",
	"2 BUG",
	"3 DANUBE",
	"4 DVORAK",
	"5 GUONOD",
	"6 MEXICO",
	"7 MOZART",
	"8 POP WEASEL",
	"9 SAM",
	"0 STARS",
	"",
	"",
	"",
	"X EXIT",
	""};


main()
{

	int idx, ch;

	int voice = 1; /* default to select voice 1 */
	int voice1 = 0;
	int voice2 = -1;
	int voice3 = -1;
	int tempo = 812;

	scr_clear(); /* clear screen */
	scr_txtcolor(C64_BWHITE);   /* bwhite text */

	/* display menus */
	printf("SONGSTER BY BILL BUCKELS 2008\n\n");
	for (idx = 0; inlist[idx][0] != '-'; idx++) {
	 printf("%-16s %-20s\n",inlist[idx], menu[idx]);

	}

	/* clear all keypressess on entry */
	while (kbhit());
	/* clear the buffer */
	kbflush();

TOP:;

   ch = toupper(getch());


   if (ch > 64 && ch < 83) {
      switch(voice) {
		  case 2:  voice2 = ch-65; break;
		  case 3:  voice3 = ch-65; break;
		  case 1:  voice1 = ch-65; break;
		  default: break;
	  }

   }
   else if (ch > 47 && ch < 58) {
	  tickvalue(tempo);
	  switch(ch) {
		  case 49: play(bingo,voice1, voice2, voice3); break;
		  case 50: play(bug,voice1, voice2, voice3); break;
		  case 51: play(danube,voice1, voice2, voice3); break;
		  case 52: play(dvorak,voice1, voice2, voice3); break;
		  case 53: play(guonod,voice1, voice2, voice3); break;
		  case 54: play(mexico,voice1, voice2, voice3); break;
		  case 55: play(mozart,voice1, voice2, voice3); break;
		  case 56: play(popwease,voice1, voice2, voice3); break;
		  case 57: play(sam,voice1, voice2, voice3); break;
		  case 48: play(stars,voice1, voice2, voice3);break;

	  }
   }
   else {
 		  switch(ch) {
			 case 'T': if (tempo > 812)tempo -= 100;  break;
			 case 'S': if (tempo < 1612)tempo += 100; break;
			 case 133: voice = 1; break;
			 case 137: voice = 2; break;
			 case 134: voice = 3; break;
			 /* reset */
			 case 138: tempo = 812;
			           unison();
                       voice = 1;
                       voice1 = 0;
                       voice2 = -1;
                       voice3 = -1;
			           break;
			 case 135: unison();   break; /* unison */
			 case 139: harmony();  break;
		  }

   }

   /* clear all keypresses */
   while (kbhit()); /* wait for release */
   /* clear the buffer */
   kbflush();

   if (ch == 'X') goto BOTTOM;

goto TOP;

BOTTOM:;

    scr_txtcolor(C64_LBLUE); /* change text back to light blue */
    /* stuff something in the keyboard to make Aztec C happy */
    kbuf[0] = 'X';
    kbuflen[0]=1;
	getchar();
	scr_clear(); /* clear screen to erase the X */

    exit(0);
}
