/* gorf.c (C) Copyright Bill Buckels 2007 */

/* This program is an Aztec C65 implementation
   of SPRITE EXAMPLE 3... THE HOT AIR GORF
   from the Commodore 64 Programmer's Reference Guide
   available for download at
   http://project64.c64.org/hw/c64.html

   It has been built under Windows XP in the
   Aztec C65 cross-development environment for the C64.

   This program links with the standard native mode
   C64NAT.LIB provided by Manx.

   Like any C64 native mode program, this program is
   loaded from BASIC 2. I have added startup and
   shutdown code so this program will exit cleanly to
   BASIC 2 when done. The original program just ran
   in a loop with no way-out.

   Other than that this is a faithful port of the
   original example from the Commodore 64 Programmer's
   Reference Guide. I have even preserved the
   ubiquituous upper case comments.

   Note: there is no need to change to graphics mode
   when using sprites nor back to text mode when done.

   */

/* THIS IS WHERE THE VIC REGISTERS BEGIN */
char *vic = (char *)53248;

char sdata[] = {
	64,0,1,16,170,4,6,170,144,10,170,160,42,170,168,41,105,104,169,
	235,106,169,235,106,169,235,106,170,170,170,170,170,170,170,170,
	170,170,170,170,166,170,154,169,85,106,170,85,170,42,170,168,10,
	170,160,1,0,64,1,0,64,5,0,80,0};

char *sarea = (char *)12288;
char *szero = (char *)2040;


/* keyboard pointers */
char *key = (char *)203;
char *kbuf = (char *)631;
char *klen = (char *)198;

main()
{
	int x, y, dx, dy, side, c;
    int c1, c2, c3, c4;

     /* set text color to white */
    printf("%c%c",5,147); /* clear text screen and home cursor */

    /* clear all keypressess on entry */
    while (key[0] != 0x40);
    /* clear the buffer */
    for (c = 9; c > -1; c--) kbuf[c]=0;
    klen[0] = 0;

    /* save orginal colors */
    c1 = vic[33];
    c2 = vic[37];
    c3 = vic[38];
    c4 = vic[39];


    /* the orginal program starts here */
    /* and never ends */

	vic[21] = 1;  	/*  ENABLE SPRITE 0	*/
	vic[33] = 14;  	/*  SET BACKGROUND COLOR TO LIGHT BLUE */
	vic[23] = 1;  	/*  EXPAND SPRITE 0 IN Y */
	vic[29] = 1;  	/*  EXPAND SPRITE 0 IN X */
	szero[0] = 192; /*  SET SPRITE 0'S POINTER */
	vic[28] = 1;  	/*  TURN ON MULTICOLOR */
	vic[37] = 7;  	/*  SET MULTICOLOR 0 */
	vic[38] = 4;  	/*  SET MULTICOLOR 1 */
	vic[0] = 100;  	/*  SET SPRITE 0'S X POSITION */
	vic[1] = 100;  	/*  SET SPRITE 0'S Y POSITION */

	vic[39] = 2;  	/*  SET SPRITE 0'S COLOR */

 	/* STORE THE DATA IN SPRITE AREA */
 	for (y=0; y<64; y++) {
    	sarea[y] = sdata[y];
 	}

    printf("PRESS 'H' FOR HI-RES\n");
    printf("PRESS 'M' FOR MULTI-COLOR\n");
    printf("PRESS 'X' TO END\n");
    printf("OTHER KEYS FOR ACTION");

    dx=1; dy=1;


 do {
  	x = vic[0]; /*  LOOK AT SPRITE 0'S X POSITION */
 	y = vic[1]; /*  LOOK AT SPRITE 0'S Y POSITION */
 	/*  IF Y IS ON THE EDGE OF THE SCREEN, THEN REVERSE DELTA Y */
    if ( y == 50 || y == 208) dy = (-dy);

 	/*  IF SPRITE IS TOUCHING THE LEFT EDGE, THEN REVERSE IT */
 	if (x == 24 && (vic[16] &1) == 0) dx = (-dx);
 	/*  IF SPRITE IS TOUCHING THE RIGHT EDGE, THEN REVERSE IT */
 	if (x == 40 && (vic[16] &1) == 1) dx = (-dx);

    /*  SWITCH TO OTHER SIDE OF THE SCREEN */
    if (x == 255 && dx == 1) {
		x = -1;
		side = 1;
	}
	/*  SWITCH TO OTHER SIDE OF THE SCREEN */
	if (x == 0 && dx == -1) {
		x = 256;
		side = 0;
	}

	/*  ADD DELTA X TO X - MAKE SURE THAT X IS IN ALLOWED RANGE */
  	x=(x+dx) &0xff;
  	/*  ADD DELTA Y TO Y */
    y=(y+dy);
  	vic[16] = side;
  	vic[0] = x;  /*  PUT NEW X VALUE INTO SPRITE 0'S X POSITION	*/
  	vic[1] = y;  /*  PUT NEW Y VALUE INTO SPRITE 0'S Y POSITION	*/


 	/* GET A KEY FROM THE KEYBOARD */
 	while ((c = key[0]) == 0x40);
 	switch(c) {
		case 23: c = 'X'; break;
		case 36: c = 'M'; break;
		case 29: c = 'H'; break;
		default: c = 0;
	}
	if (c== 'X') break;
	if (c != 0) while (key[0] != 0x40);

 	if (c=='M') vic[28] = 1;  /*  USER SELECTED MULTICOLOR */
 	if (c=='H') vic[28] = 0;  /*  USER SELECTED HIGH RESOLUTION */

 } while (c != 'X');


 /* clear all but the last keypress */
 while (key[0] != 0x40);
 /* fix the buffer so it has the last character pressed */
 for (c = 9; c > 0; c--) kbuf[c]=0;
 kbuf[0] = 'X';
 klen[0] = 1;

 vic[28] = 0;  	/*  TURN OFF MULTICOLOR */
 vic[21] = 0;  	/*  DISABLE SPRITE 0	*/

 /* restore original colors */
 vic[33] = c1;
 vic[37] = c2;
 vic[38] = c3;
 vic[39] = c4;


 /* Aztec C needs to read and dispose of the last keypress
    in order to return cleanly to BASIC 2 */
 c = getchar();

 /* set text color to light blue */
 printf("%c%c",154,147); /* clear */

 exit(0);

}








