/* VECTOR.c (C) Copyright Bill Buckels 2007  */
/* written in Manx Aztec C65 Version 1.05h   */
/* runs under Commodore BASIC 2              */

/* This is a cumputationally intensive demo
   and not really very realistic for a real C64
   because it runs too slow.

   I wrote this to demonstrate some of the graphics
   functions that I created for B64NAT.LIB but in
   reality you will want to use these sparingly
   when needed unless you are writing a paint program.

   These aren't heavily optimized because I wanted to keep
   them fairly simple and readable but many standard
   techniques could have been applied including operating
   on blocks of data where possible, fewer stack variables
   and temporary variables, etc. but in my opinion at
   the expense of readability.

   You should optimize these before using them as a core
   for anything but a small amount of vector graphics
   and just use my functions as a guideline. There is
   some optimization in mine but more of a general purpose
   nature than anything.

   Anyway, run this demo in warpmode if you are using
   an emulator. */



/* memory map for creating C64 graphics programs in Aztec C

   0x800 - (2048) - basic load address
   0x810 - binary base address
   0x4000 - data load address (or higher if you want additional room
                               below this area to store other static
                               chunks of data)
   0x4400 - code load address (or higher if lots of data)

   for embedding a HIRES image
   insert vram before bitmap

   1000 bytes for vram storage
   8000 bytes for bitmap

   insert bitmap at default screen address when program loads
   insert vram 1K before bitmap
   0x2000
   0x1c00

   the bitmap is immediately displayed on the screen and vram
   storage is then moved immediately to the actual vram address
   (use setlogo in B64NAT.LIB)

   for embedding a 1024 byte 8 x 8 fontset (characters 0-127)
   insert fontset 2K before bitmap
   0x1800

   I have found in practice that no insertion of data can be done
   below these areas.


*/

/* the CMAIN define must always be the first line
   in your main program when
   using poke.h and B64NAT.LIB and nowhere else.

   B64NAT.LIB makes extensive use of the memory variables
   in poke.h which are included in your main program
   and nowhere else.

   They are referenced as externals by B64NAT.LIB and any
   overlay modules that you may create.

   when linking with B64NAT.LIB the aztec C linker requires
   B64NAT.LIB on the link line before C64NAT.LIB to
   resolve B64NAT.LIB calls to C64NAT.LIB
*/

#define CMAIN 1

#include <poke.h>
#include <colors.h>

char *warp   = "Run in Warpmode";
char *prompt = "Press Any Key!";
char *pclear = "               ";

main()
{
	int x,y, color, color1, color2, color3, row = 12, col = 13;

    scr_clear();

    /* border color,background color */
    scr_back(C64_BLACK,C64_BLUE);

	/* set to hires mode 320 x 200 and display the logo */
    setcrtmode(1);
    setlogo();

    pfont(warp,21,24,C64_LBLUE,C64_BWHITE,-1);
	pfont(prompt,22,24,C64_LBLUE,C64_BWHITE,-1);

    kbflush();
    getch();

    /* clear bmp, vram, and cram colors*/
    gclr(0,16,0);

    color = 1;

    /* draw some boxes */
    x = 0;
    for (y = 0; y < 92; y+=3) {
        box(x,y,319-x,199-y,color,1);
     	x+=3;
	}

    pfont(prompt,row,col,C64_BLACK,C64_BWHITE,-1);

	kbflush();
	getch();

    pfont(pclear,row,col,C64_BLACK,C64_BWHITE,-1);

	color = 0;

    x = 0;
    for (y = 0; y < 92; y+=3) {
         box(x,y,319-x,199-y,color,1);
      	 x+=3;
 	}

    /* draw some disks */
	disk(160,100,99,1,1);
    pfont(prompt,row,col,C64_BLACK,C64_BWHITE,-1);

	kbflush();
	getch();

	disk(160,100,99,0,1);

    /* draw some circles */
	color = 1;

    for (x = 99; x > 50; x-=3) {
       circle(160,100,x,color,1);
	}


    pfont(prompt,row,col,C64_BLACK,C64_BWHITE,-1);

	kbflush();
	getch();

	color1 = C64_CYAN;
	color2 = C64_PURPLE;
	color3 = C64_BWHITE;

    color = (color1 << 4 | color2);

    gclr(0,color,color3);

	/* set multicolor graphics mode 160 x 200 */
    setcrtmode(2);

    gclr(0,-1,-1);

    /* fill some boxes */
    fbox(0,0,24,39,1,C64_PURPLE,C64_CYAN, C64_BWHITE,2);
    fbox(4,8,20,31,2,C64_PURPLE,C64_CYAN, C64_BWHITE,2);
    fbox(8,16,16,23,3,C64_PURPLE,C64_CYAN, C64_BWHITE,2);

	kbflush();
	getch();

    gclr(0,-1,-1);

    color = 1;

    /* draw some boxes */
    x = 0;
    for (y = 0; y < 92; y+=4) {
        box(x,y,159-x,199-y,color,2);
     	x+=2;
	}

    pfont(prompt,row,col,C64_BLACK,C64_CYAN,C64_CYAN);

	kbflush();
	getch();

	pfont(pclear,row,col,C64_BLACK,C64_CYAN,C64_CYAN);

    color = 2;

    x = 0;
    for (y = 0; y < 92; y+=4) {
        box(x,y,159-x,199-y,color,2);
     	x+=2;
	}

    pfont(prompt,row,col,C64_BLACK,C64_PURPLE,C64_PURPLE);

	kbflush();
	getch();

	pfont(pclear,row,col,C64_BLACK,C64_PURPLE,C64_PURPLE);

    color = 3;

    x = 0;
    for (y = 0; y < 92; y+=4) {
        box(x,y,159-x,199-y,color,2);
     	x+=2;
	}

    pfont(prompt,row,col,C64_BLACK,C64_BWHITE,C64_BWHITE);

	kbflush();
	getch();

	pfont(pclear,row,col,color2,color1,color3);

    color = 0;

    x = 0;
    for (y = 0; y < 92; y+=4) {
        box(x,y,159-x,199-y,color,2);
     	x+=2;
	}

    /* draw some disks */
    for (color = 1; color < 4; color++)disk(80,100,49,color,2);

    disk(80,100,49,0,2);


    /* draw some circles */
    while (kbhit());
    for (color = 1; color < 4; color++) {
	      for (x = 49; x > 10; x-=3) {
			  if (kbhit())
				  x = 0;
			  else
		          circle(80,100,x,color,2);
	      }
	      if (kbhit() || x == 0)break;
	}

	pfont(prompt,row,col,C64_BLACK,C64_CYAN,C64_CYAN);
	kbflush();

	while (!kbhit()); /* leave the last key in the buffer for
	                     the call to getchar() below. */

    /* clear last palette and bitmap */
    gclr(0,0,0);

    /* set to text mode */
    setcrtmode(0);

    /* always call getchar at least once to initialize aztec C's IO
       buffers or your program will hang the system when it exits
       this is just the way Aztec operates on the C64 and there is
       no point worrying about it.*/

    /* it's nice and clean to call this at the very end
       and don't bother anywhere else if you don't need to
       it messes up the screen in graphics modes and is pretty
       much undesirable in text modes since it leaves your program
       open to user input of screen control characters
       and it displays an ugly cursor */
    getchar();
    /* always flush the keyboard before exiting to basic */
    kbflush();
    scr_clear();

	exit(0);
}

