/*	Copyright (c) 1982, 1983 by Manx Software Systems	*/
/*	Copyright (c) 1982, 1983 by Jim Goodnow II		*/

#ifndef CPM
#include	"kbctl.h"
#endif

/*
	nm - produce a name list (symbol table) from the specified object files.

	nm [-anuog] file1 [file2] ...
		or
	nm -s file1 [file2] ...
*/

#define H_MAGIC	0x1a04

struct header {
	int h_magic;		/* magic number for object files */
	char h_name[8];		/* module name */
	unsigned h_code;	/* size of code in this module */
	unsigned h_data;	/* size of data in this module */
	unsigned h_static;	/* size of static storage in this module */
	unsigned h_local;	/* offset of start of local symbols */
	unsigned h_global;	/* offset of start of global symbols */
	unsigned h_end;		/* end of local symbols */
	unsigned h_next;	/* offset of next module in this file */
};

#define S_UND		7
#define S_TOKEN		127
#define S_GLOBL 	0x08

struct gbltab {
	char g_type,g_flags;
	unsigned g_value;
	int	g_common;
	char g_name[8];
};

char flags[] = " ATDCREUWX";
int sizeflg, namflg, outflg, undflg, glbflg, alphflg, numflg;

main(argc, argv)
register int argc;
register char *argv[];
{
	struct header hdr;
	register struct gbltab *glb, *bglb, *eglb;
	register struct gbltab **glbs;
	long lseek();
	int i, fd, nglb;
	register char *cp;
	long ttot, tcode, tdata, tstatic;
	unsigned tot, cnt = 0;
	int width;

#ifdef CPM
	width = 4;
#else
	if (ioctl(1, KB_STAT))
		width = 1;
	else if (ioctl(1, KB_WID) == 80)
		width = 4;
	else
		width = 2;
#endif
	if (argc < 2) {
		printf("usage: nm [-anuog] file1 [file2] ...\n");
		printf("	or\n");
		printf("usage: nm -s file1 [file2] ...\n");
		exit(1);
	}
	sizeflg = alphflg = numflg = namflg = outflg = undflg = glbflg = 0;
	ttot = tcode = tdata = tstatic = 0;
	glbs = alloc(1500 * 2);
	bglb = alloc(1500 * sizeof(struct gbltab));
	while (--argc) {
		argv++;
		if (**argv == '-') {
			cp = *argv;
			while (*++cp) {
				switch(tolower(*cp)) {
				case 'u':
					undflg++;
					break;
				case 'o':
					outflg++;
					break;
				case 'g':
					glbflg++;
					break;
				case 'a':
					alphflg++;
					break;
				case 'n':
					numflg++;
					break;
				case 's':
					sizeflg++;
					break;
				default:
					printf("Bad option - `%c'.\n", *cp);
					break;
				}
			}
			continue;
		}
		if (namflg == 0)
			if (argc > 1)
				namflg++;
		if ((fd = open(*argv, 0)) < 0) {
			printf("Error opening %s.\n", *argv);
			continue;
		}
		if (read(fd, &hdr, sizeof (hdr)) <= 0) {
			printf("Error reading %s.\n", *argv);
			close(fd);
			continue;
		}
		if (hdr.h_magic != H_MAGIC) {
			printf("File %s not in proper format.\n", *argv);
			close(fd);
			continue;
		}
		if (sizeflg) {
			tot = hdr.h_code + hdr.h_data + hdr.h_static;
			tcode += hdr.h_code;
			tdata += hdr.h_data;
			tstatic += hdr.h_static;
			ttot += tot;
			cnt++;
			printf("%s: %u+%u+%u = %u = $%x\n", *argv, hdr.h_code,
										hdr.h_data, hdr.h_static, tot, tot);
			close(fd);
			continue;
		}
		if (lseek(fd, (long) hdr.h_global, 0) < 0) {
			printf("Error seeking in %s.\n", *argv);
			close(fd);
			continue;
		}
		if (namflg)
			printf("\n%s:\n", *argv);
		nglb = hdr.h_local - hdr.h_global;
		if ((i = read(fd, bglb, nglb)) != nglb) {
			printf("Error reading symbol table in file %s.\n", *argv);
			close(fd);
			continue;
		}
		nglb /= sizeof(struct gbltab);
		eglb = bglb + nglb;
		for (i=0,glb=bglb;glb < eglb;i++,glb++)
			glbs[i] = glb;
		if (alphflg)
			asort(glbs, nglb);
		if (numflg)
			nsort(glbs, nglb);
		for (i=0;i<nglb;i++) {
			dump(glbs[i], *argv);
			if (outflg || (i+1)%width == 0)
				printf("\n");
		}
		printf("\n");
		close(fd);
	}
	if (sizeflg && cnt > 1)
		printf("Total: %lu+%lu+%lu = %lu = $%lx\n", tcode,
											tdata, tstatic, ttot, ttot);
}

nsort(glbs, cnt)
struct gbltab *glbs[];
register int cnt;
{
	register int i, j, k;
	register struct gbltab *tglb;

	for (i=0;i<cnt-1;i++) {
		k = i;
		for (j=i+1;j<cnt;j++) {
			if (glbs[j]->g_value < glbs[k]->g_value)
				k = j;
		}
		tglb = glbs[i];
		glbs[i] = glbs[k];
		glbs[k] = tglb;
	}
}

asort(glbs, cnt)
struct gbltab *glbs[];
register int cnt;
{
	register int i, j, k;
	register struct gbltab *tglb;

	for (i=0;i<cnt-1;i++) {
		k = i;
		for (j=i+1;j<cnt;j++) {
			if (strncmp(glbs[j]->g_name, glbs[k]->g_name, 8) < 0)
				k = j;
		}
		tglb = glbs[i];
		glbs[i] = glbs[k];
		glbs[k] = tglb;
	}
}

dump(glb, file)
register struct gbltab *glb;
register char *file;
{
	char name[10];

	if (glb->g_type == S_TOKEN)
		glb->g_type = 9;
	strncpy(name, glb->g_name, 8);
	name[8] = 0;
	if (undflg && glb->g_type == S_UND) {
		if (outflg)
			printf("%s: ", file);
		printf("     %c %-12.12s", flags[glb->g_type], name);
		return;
	}
	if ((glbflg && (glb->g_flags&S_GLOBL)) || (undflg == 0 && glbflg == 0)) {
		if (outflg)
			printf("%s: ", file);
		printf("%04x %c %-12.12s", glb->g_value, flags[glb->g_type], name);
		return;
	}
}

