/*	Copyright (C) 1984 by Manx Software Systems, Inc.		*/

#include	"xfer.h"
#include	"stdio.h"

sendfile(name)
char *name;
{
	FILE *fp;
	register int i, len, l, id;
	register char *cp;
	int retry = 0;
	char buf[5*1024];

	id = 1;
	fp = fopen(name, "r");
	if (fp == NULL) {
		print("\nFailure opening <%s>.\n", name);
		abort(1);
		return(-1);
	}
	print("\nSending <%s>.\n", name);
	for (;;) {
		if (id == 1 && Master && Progflg) {
			len = fread(buf, 1, 4, fp);
			len += fread(buf+2, 1, 5*1024-2, fp) - 2;
		}
		else if (Asciiflg) {
			for (len=0;len<5*1024;len++) {
				if ((i = agetc(fp)) == EOF)
					break;
				buf[len] = i;
			}
		}
		else
			len = fread(buf, 1, 5*1024, fp);

		if (len == 0) {
			if ((i = psend(buf, ALLDONE, id, 0)) == -1) {
				fclose(fp);
				print("\nFailure sending EOT packet.\n");
				abort(1);
				return(-1);
			}
			if (i == -2) {
				fclose(fp);
				print("\nAbort by receiver.\n");
				return(-1);
			}
			retry += i;
			print(".");
			fclose(fp);
			print("\nSent %d packets, %d retries.\n", id, retry);
			return(0);
		}
		if (len < 0) {
			fclose(fp);
			print("\nFailure reading file.\n" );
			abort(1);
			return(-1);
		}
		cp = buf;
		while (len) {
			if (len > 128)
				l = 128;
			else
				l = len;
			if ((i = psend(cp, SENDDAT, id, l)) == -1) {
				fclose(fp);
				print("\nFailure sending data packet %d.\n", id);
				abort(1);
				return(-1);
			}
			if (i == -2) {
				fclose(fp);
				print("\nAbort by receiver.\n");
				return(-1);
			}
			retry += i;
			print(".");
			cp += l;
			len -= l;
			id++;
		}
	}
}

recvfile(name)
char *name;
{
	FILE *fp;
	char *cp;
	int typ, id, len, l, packet;
	char buf[5*1024];

#ifdef APPLE
	fp = fopen(name, Asciiflg?"w":"wb");
#else
	fp = fopen(name, "w");
#endif
	if (fp == NULL) {
		print("\nFailure creating file <%s>.\n", name);
		abort(0);
		return(-1);
	}
	print("\nReceiving <%s>.\n", name);
	packet = typ = 0;
	while (typ != ALLDONE) {
		len = 0;
		while (len+128 <= 5*1024 && typ != ALLDONE) {
			precv(buf+len, &typ, &id, &l);
			if (typ == ABORT) {
				fclose(fp);
				print("\nAbort by sender.\n");
				return(-1);
			}
			if (++packet != id) {
				fclose(fp);
				print("\nBad packet id - wanted %d, got %d.\n", packet, id);
				abort(0);
				return(-1);
			}
			print(".");
			len += l;
		}
		if (Asciiflg) {
			cp = buf;
			while (cp<buf+len) {
				if (aputc(*cp++, fp) == EOF) {
					fclose(fp);
					print("\nFailure writing file.\n");
					abort(0);
					return(-1);
				}
			}
		}
		else if (fwrite(buf, 1, len, fp) != len) {
			fclose(fp);
			print("\nFailure writing file.\n");
			if (typ != ALLDONE)
				abort(0);
			return(-1);
		}
	}
	fclose(fp);
	print("\nReceived %d packets.\n", id);
	return(0);
}

/*
 *	packet send and receive protocol
 */

psend(msg, type, id, len)
char *msg;
int len, type, id;
{
	int retry, c, i, cnt;
	char *cp;
	char buf[134];

	cp = buf;
	*cp++ = type;
	*cp++ = id;
	*cp++ = id >> 8;
	*cp++ = len;
	for (c=0;c<128;c++)
		*cp++ = msg[c];
	Crclow = Crchigh = 0;
	for (c=0;c<132;c++)
		acumcrc(buf[c]);
	*cp++ = Crchigh;
	*cp++ = Crclow;
	setvec();
	for (retry=0;retry<10;retry++) {
		cnt = 134;
		cp = buf;
		for (;;) {
			c = cget();
			if (c == GONEW) {
#ifdef DEBUG
				debug("N");
#endif
				if (retry == 0)
					break;
				clrvec();
				return(retry-1);
			}
			if (c == GOSAME) {
#ifdef DEBUG
				debug("+");
#endif
				break;
			}
			if (c == ABORT) {
				dput(ABORT);
				clrvec();
				return(-2);
			}
#ifdef DEBUG
			debug("w");
#endif
		}
		for (c=0;c<100;c++)
			;
		while (cnt--)
			dput(*cp++);
	}
	clrvec();
	return(-1);
}

precv(msg, type, id, len)
char *msg;
int *len, *type, *id;
{
	int l, c, cnt, w = GONEW;
	char *cp;
	char buf[134];

	setvec();
	for (;;) {
#ifdef DEBUG
		debug("r");
#endif
		cnt = 133;
		cp = buf;
		while ((c = cget()) == -1)
			dput(w);
		*cp++ = c;
		while (cnt--)
			*cp++ = cget();
		Crclow = Crchigh = 0;
		for (l=0;l<134;l++)
			acumcrc(buf[l]);
		if (Crclow + Crchigh != 0) {
#ifdef DEBUG
			debug("+");
			clrvec();
			print("\n");
			for (l=0;l<134;l++)
				print("%02x  ", buf[l]&0xff);
			print("\n");
			setvec();
#endif
			dput(w=GOSAME);
			continue;
		}
		cp = buf;
		*type = *cp++ & 0xff;
		*id = *cp++ & 0xff;
		*id |= *cp++ << 8;
		*len = *cp++ & 0xff;
		for (l=0;l<*len;l++)
			msg[l] = *cp++;
		dput(GONEW);
		clrvec();
		return;
	}
}

abort(flg)
int flg;
{
	print("ABORTING\n");
	if (flg)
		psend(&flg, ABORT, 0, 0);
	else {
		while (cget() == -1)
			dput(ABORT);
	}
}

dput(c)
int c;
{
	int i;

	for (i=0;i<Delay;i++)
		;
	cput(c);
}

/*	CRC16	*/

acumcrc(v)
{
	register unsigned a,p,x;

	x = a = (Crchigh ^ v) & 0xff;
	for (p = 0 ; x ; ) {
		p ^= x&1;
		x >>= 1;
	}
	if (p) {
		Crchigh = ((a&1)?0x80:0) ^ ((a<<6)&0xc0) ^ 0x01 ^ Crclow;
		a >>= 1;
		Crclow = (0x80>>1) ^ (a>>1) ^ a ^ 0x80;
	} else {
		Crchigh = ((a&1)?0x80:0) ^ ((a<<6)&0xc0) ^ Crclow;
		a >>= 1;
		Crclow = (a>>1) ^ a;
	}
}

quit()
{
	exit(1);
}

pchar(c)
int c;
{
#ifdef UPPER
	if (c >= 'a' && c <= 'z')
		c = c - 'a' + 'A';
#endif
	putchar(c);
}

print(msg, a1, a2, a3)
char *msg;
{
	format(pchar, msg, &a1);
}

#ifndef C64
setvec()
{
}
clrvec()
{
}
#endif

#ifdef DEBUG
debug(str, a1, a2, a3, a4, a5)
char *str;
{
	clrvec();
	print(str, a1, a2, a3, a4, a5);
	setvec();
}
#endif

