
/* Frag64 (C) Copyright 2008 Bill Buckels */

/* converts from an IBM Bsaved PUT to a C64 image fragment */
/* fullscreen PIC's convert to BSAVED C64 Images with Colors */
/* extensions are fr2 or fr4 for fragments */
/* extensions are BHI or BMC for BSAVED C64 Images */
/* Color File Extensions are VHI or VMC and CMC */
/* The full screen files are in the Clipshop format */

/* The FRAGGLE.EXE utility will produce PUT format images from CGA
   4-Color BSaved Files which are freely available on the IBM-PC.
   Clipshop can be used to create 4 Color BSaved Files from the Windows
   Clipboard and other sources as can several old utilites.

   To learn more about all this visit my wikipedia page at

   http://en.wikipedia.org/wiki/BSAVE_(graphics_image_format)

   Generally speaking you are better-off to use FRAGGLE.EXE
   menu option 5 to create the PUT image fragments that you will
   be using in your C64 programs because these are saved in 8 x 8
   graphics blocks which is perfect for C64 conversion.


   Note that when converting from a 4 color image to a HIRES
   image on the C64, any color other than black is taken as white.

   Also note that when converting from a 4 color image to a
   Multicolor image on the C64, every second pixel is skipped
   since the horizontal resolution is only 160 pixels in Multicolor
   mode. This means that you should use lines of 2 pixels in
   width for your line elements and stay away from altrenating pixel
   pattern files. Solid color works best in colored areas.

   If you want to preserve detail in a Multicolor mode image
   you will need to draw it as a 160 x 100 picture on the
   IBM then scale it x 2 and BSAVE it before using FRAGGLE.EXE.

   If you want your text and graphics to be near perfect in the
   horizontal axis in a Multicolor image that is what you must do,
   and for any menu screens that is what I would suggest. Your
   line elements in your artwork will be somewhat jagged but your
   results will be quite manageable for combining fragments
   with background screens and a common palette.

   I consider the sprites on a C64 a little small for my preferences
   on this type of programming, and I much prefer a clean image
   fragment instead even if I need to reduce to 4 colors.

   Lastly, this utility provides a command line override for
   mapping to your choice of each of the C64 colors 0-15 for
   your full screens.

   This gives you a choice of any 2-colors in your HIRES images
   and any 4-colors in your Multicolor images, which I feel provides
   a reasonable amount of flexibility in choosing a color scheme
   to combine fragments and screens.

   You have this source code if you want to customize this utility
   further. The 8 x 8 color block on a C64 makes converting a 4 color
   image a little tricky and as a programmer's tool I have gone about
   as far as I think is generally reasonable, but many more schemes
   are possible even with this lot. For example, widening your black
   lines or white lines on line-art in a multicolor image will
   guarantee that detail is not lost in some cases. In others it will
   destroy your graphics by creating blotches

   For these types of things hotkeys are used in this program.
   You can read the source to see what they are and try them
   for yourself. I stopped short of scaling and so forth and again
   you can add these functions yourself if you like.

   On a further note...

   Clipshop has more options for general purpose conversion of full
   screens but combining those with fragments will likely prove
   problematic, and even Clipshop conversions will prove problematic
   on their own, since color precedence is used to map the most used
   colors and colors are not what and where you might expect them
   from block to block and will be lost altogether if a complex
   image and no spot is available.

   Much better to deal with less colors and guaranteed results with
   this utility is my conclusion over the years of mucking with all
   this is what I think. Do your fancy stuff in a paint program
   first and use this utility to process the finished artwork is
   what I suggest.

   'Nuff Said.

   Download Clipshop at

   www.clipshop.ca

   */

#include <stdio.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include <dos.h>
#include <bios.h>
#include <malloc.h>
#include <string.h>
#include <graph.h>

#define bitsperbyte 8

unsigned int tempwidth, tempheight;

char vhi[66], cmc[66], vmc[66];

int Color1, Color2, Color3, background;

/* poke values for colors */
/* not used - left here for reference */
#define C64_BLACK 0
#define C64_BWHITE 1
#define C64_RED	2
#define C64_CYAN 3
#define C64_PURPLE 4
#define C64_GREEN 5
#define C64_BLUE 6
#define C64_YELLOW 7
#define C64_ORANGE 8
#define C64_BROWN 9
#define C64_LRED 10
#define C64_DGRAY 11
#define C64_MGRAY 12
#define C64_LGREEN 13
#define C64_LBLUE 14
#define C64_LGRAY 15

/* return the color value of the pixel at x, y */
unsigned char getpixel(int x, int y)
{
    union REGS inregs, outregs;

    inregs.h.ah = '\x0d';
    inregs.x.cx = x;
    inregs.x.dx = y;

    int86(0x10, &inregs,&outregs);
    return outregs.h.al;
}

void putpixel(int x, int y,int pixelvalue)
{
    union REGS inregs, outregs;

    inregs.h.ah = '\x0c';
    inregs.h.al = pixelvalue;
    inregs.x.cx = x;
    inregs.x.dx = y;

    int86(0x10, &inregs,&outregs);
}

int fragin(char *fragment)
{

     FILE *fp;

     int fh,action,x,y;
     unsigned char far *buffer;
     unsigned char far *crt= (char *)0xb8000000l;
     unsigned int width, height;
     unsigned int buffersize,headerlength=7;
     unsigned char floodcolor;
     char c;

    fp=fopen(fragment,"rb");

    if (NULL == fp)return 1;

    buffersize=filelength(fileno(fp))-headerlength;
    fclose(fp);

    buffer=_fmalloc(buffersize);

    if (NULL == buffer)return 1;

    fh = open(fragment,O_RDONLY|O_BINARY);
    read(fh,buffer,7);

    /* minimal header check */
    /*
    if (buffer[0] != (char)0xfd) {
		close(fh);
		return 1;
	}
	*/

    read(fh,buffer,buffersize);
    close(fh);

   	width=(buffer[0]|buffer[1]<<8) ;

   	tempwidth=width/2;

   	if((width%bitsperbyte)!=0)
   		width=(width/bitsperbyte)+1;
   	else
   		width=(width/bitsperbyte) ;

   	height=(buffer[2]|buffer[3]<<8);
   	tempheight = height;

   /*
   floodcolor=buffer[4];
   memset(crt,floodcolor,0x4000);

   */

    _putimage(0,0,buffer,_GPSET);

    while ((c = toupper(getch()))!=13) {

       if (c == 'B')floodcolor = 0;
       if (c == 'C')floodcolor = 1;
       if (c == 'M')floodcolor = 2;
       if (c == 'W')floodcolor = 3;

       if (c == 'A') action = _GAND;
       if (c == '0') action = _GOR;
       if (c == 'P') action = _GPRESET;
       if (c == 'R') action = _GPSET;
       if (c == 'X') action = _GXOR;

		switch(c)
		{
			case 0:     getch(); break; /* fkeys are not valid here */
			case 'X':
			case 'R':
			case 'P':
			case 'O':
			case 'A':
					   _putimage(0,0,buffer,action);
					   c=getch();
					   if (c == 0)getch();
					   if (c == 27) _putimage(0,0,buffer,_GPSET);
					   break;
			case 'S':  _putimage(1,0,buffer,_GPSET);
					   c=getch();
					   if (c == 0)getch();
					   if (c == 27) _putimage(0,0,buffer,_GPSET);
					   break;

            case 'B':
            case 'C':
			case 'M':
			case 'W':  for(y = 0; y < 200; y++)
						for (x = 1; x < 320; x+=2)
						   if (getpixel(x,y) == floodcolor)
							  putpixel(x-1,y,floodcolor);

					   c=getch();
					   if (c == 0)getch();

					   if (c == 27) _putimage(0,0,buffer,_GPSET);
					   break;


		}

    }

    _ffree(buffer);
    return 0;

}




/* saves a hires C64 image */
int savefrag64(char *name, int mode)
{

FILE *fp;
int x, y, x1, y1, rows, columns, row, col;
unsigned c, ch;

	if(tempwidth%8 !=0)
	  tempwidth  = ((tempwidth/8)*8)+8; /* multiple of 8 */

	if(tempheight%8 !=0)
	  tempheight  = ((tempheight/8)*8)+8; /* multiple of 8 */

	columns = tempwidth/8;
	rows = tempheight/8;

	fp = fopen(name,"wb");

	if (NULL == fp) {
		/* perror(name); */
		return 1;
	}

   /* create a header */
   /* meaning is rows x columns if a fragment */

   /* image is block oriented */
   if (tempwidth == 320 && tempheight == 200) {
	   fputc(0,fp);
	   fputc(0x20,fp); /* if a full screen, image is bsaved with colors */
   }
   else {
	   /* if a fragment image is saved without colors */
	   fputc(rows,fp); /* this is oriented to how the loader works */
	   fputc(columns,fp); /* and to how the C64 screen is arranged */
   }

   for (row = 0; row < rows; row++) {
	   for (col = 0; col < columns; col++) {

		   y = row * 8;
		   x = col * 8;

	       for (y1 = y; y1 < (y+8); y1++) {
			  c = 0;
		      for (x1 = x; x1 < (x+8); x1+=mode) {
			       c = c << mode;
			       if (mode == 1) {
					   if (getpixel(x1,y1) != 0)c = c|1; /* map colors to white */
				   }
			       else {
					   ch = getpixel(x1,y1);
					   c = c | ch;
				   }
			  }
			  if (mode == 1)c = c^0xff;
			  fputc(c,fp);
		  }

	   }
   }




   fclose(fp);

   /* if a full screen, save the colors */
   if (tempwidth == 320 && tempheight == 200) {

	   switch(mode) {
	       case 2:   fp = fopen(vmc,"wb");
	                 if (NULL == fp) break;
	                 fputc(0,fp);
					 fputc(0x1C,fp);
					 /* light blue and light red defaults */
	                 c = (Color1 << 4 | Color2);
 					 for (x = 0; x <1000; x++)fputc(c,fp);
	                 fclose(fp);

                     fp = fopen(cmc,"wb");
	                 if (NULL == fp) break;
	                 fputc(0,fp);
					 fputc(0xD8,fp);
		             c = (background << 4 | Color3);
	                 for (x = 0; x <1000; x++)fputc(c,fp); /* default white colormap */
	                 fclose(fp);
	                 break;


	       default:  fp = fopen(vhi,"wb");
	                 if (NULL == fp) break;
	                 fputc(0,fp);
					 fputc(0x1C,fp);
					 c = (Color2 << 4 | Color1);
					 for (x = 0; x <1000; x++)fputc(c,fp);
					 fclose(fp);
					 break;


	   }


   }
   return 0;

}

main(int argc,char *argv[])
{
   int result,mode;
   char buf1[66],buf2[66], ext[5];
   char *wordptr;

   if (argc < 2) {
      puts("Frag64 (C) Copyright 2008 Bill Buckels");
      puts("All Rights Reserved.");
      puts("Usage is \"FRAG64 MY.PUT\"");
      puts("Option   \"FRAG64 MY.PUT 2\"");
      puts("Option   \"FRAG64 MY.PUT 4\"");
      puts("Options   Number of Colors (2 or 4)");
      puts("Default   2 Colors");
      puts("Input     IBM-PC 4 Color CGA BSaved Image Fragment");
      puts("Output    Commodore 64 HIRES or Multi-Color Image Fragent");
      puts("Extension MY.fr2 (2 Color) or MY.fr4 (4 Color) for fragments");
      puts("Extension MY.BHI (2 Color) or MY.BMC (4 Color) for Full Screen");
      puts("Option 2  Color Mapping (Default is B&W (hires) or BRW on Black)");
      puts("          Color Map Values are in the range of 0-15");
      puts("Option 2  \"FRAG64 MY.PUT 2 Color1 background\"");
      puts("Option 2  \"FRAG64 MY.PUT 4 Color1 Color2 Color3 background\"");
      puts("          Color Mapping is for Fullscreen Only (320 x 200)");
      exit(result);

   }

   /* if we are saving a hires fragment
      we encode every pixel.

      if we are saving a multicolor fragment
      we encode every second pixel.

      in either case the background is assumed
      to be black for the entire image.

      this makes CGA mode fragments ideal for
      encoding both screens and fragments.

      any palette that we wish to set in our
      programs can be mapped to each of the
      available colors in real time.

      a consistent palette needs to used anyway
      if we are combining screens and fragments.

      */

     strcpy(ext,".fr2");
     mode = 1;

     Color1 = 1;
     Color2 = 0;
     Color3 = 1;
     background = 0;


     if (argc > 2) {
		 if (argv[2][0] == '4') {
			 strcat(ext,".fr4");
			 mode = 2;
             Color1 = 14; /* lblue */
             Color2 = 10; /* lred  */
		 }
		 if (argc > 3) Color1 = atoi(argv[3])&0xf;
		 if (argc > 4) Color2 = atoi(argv[4])&0xf;
		 if (argc > 5) Color3 = atoi(argv[5])&0xf;
		 if (argc > 6) background = atoi(argv[6])&0xf;
	 }

	_setvideomode(5); /* CGA 4 Color */
	result = fragin(argv[1]);

	if (result == 0) {

		/* full screen */
		if (tempwidth == 320 && tempheight == 200) {
			if (mode == 1)  strcpy(ext,".bhi");
			else strcpy(ext,".bmc");
		}

		strcpy(buf1,argv[1]);
		wordptr=strtok(buf1,".");
		sprintf(buf2,"%s%s",buf1,ext);
		sprintf(vhi,"%s.vhi",buf1);
		sprintf(cmc,"%s.cmc",buf1);
		sprintf(vmc,"%s.vmc",buf1);

		result = savefrag64(buf2,mode);

    }
	_setvideomode(3); /* CGA 80 Col Color Text */

    exit(result);
}
