
/* sndfrag.c (C) Copyright Bill Buckels 2008 */
/* converts one binary musical file of type .SND   */
/* which are IBM files containing frequency, duration pairs in IBM format */
/* to several character arrays for embedding in Commodore 64 C code */

/* if space is pressed the next note plays */
/* if enter is pressed the current array is finished and a new one is started */
/* if escape is pressed the last array is closed and removed from disk */
/* and the process ends */

/* these fragments of the original file can be combined with */
/* graphics image fragments for an animation and sound sequence */
/* see the "What Time Is It" program in the projects directory for an example */


#include <malloc.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>

#include <dos.h>

#define TIMEOUT 0x2c00

/* freq of sound in Hertz                  */
/* duration of sound 18.2 ticks per second */

void sound(int freq, int tlen)
{
  union REGS inregs, outregs;
  unsigned frdiv;
  long lDiv;
  int seed, hiseed, hold=0, setting;

  if (tlen < 1)
    return;

  if (freq != 32767) {
    lDiv = 1331000L/freq;        /* timer divisor */
    frdiv = (unsigned)lDiv;
    outp(0x43, 0xB6) ;           /* write timer mode register */
    outp(0x42, frdiv & 0x00FF);  /* write divisor LSB */
    outp(0x42, frdiv >> 8);      /* write divisor MSB */
    setting = inp(0x61);         /* get current port B setting */
    outp(0x61,setting | 0x03);   /* turn speaker on */
  }

  if(tlen > 0) {
    tlen = ((tlen*1000) /182); /* convert from 18.2 ticks to 100ths */
    inregs.x.ax = TIMEOUT;
    int86(0x21, &inregs, &outregs);
    seed = (outregs.x.dx)&0xff;

    while(hold<tlen) {
      inregs.x.ax = TIMEOUT;
      int86(0x21,&inregs,&outregs);
      if(seed!=(outregs.x.dx&0xff)) {
        hiseed= (outregs.x.dx)&0xff;
        if (hiseed < seed)
          hiseed+=100;
        hold+=(hiseed-seed);
        seed =(outregs.x.dx)&0xff;
      }
    }
  }

  /* restore port B setting */
  if (freq != 32767)
    outp(0x61,setting);
  return;
}


/* ------------------------------------------------------------------- */
/* Translates a frequency within a range to a corresponding midi note. */
/* ------------------------------------------------------------------- */
int xplay(int xfrequency)
{

  /* octaves out of C64 range are ignored */
  if(xfrequency < 32 || xfrequency > 8192 )
  {
    /* if we are out of range don't play anything */
    return -1;
  }

  /* octave 0 */
  if(xfrequency<63)
  {
  if(xfrequency<34 ) return  12;
  if(xfrequency<36 ) return  13;
  if(xfrequency<38 ) return  14;
  if(xfrequency<40 ) return  15;
  if(xfrequency<42 ) return  16;
  if(xfrequency<45 ) return  17;
  if(xfrequency<48 ) return  18;
  if(xfrequency<51)  return  19;
  if(xfrequency<54)  return  20;
  if(xfrequency<57)  return  21;
  if(xfrequency<60)  return  22;
                     return  23;
  }

  /* first octave */
  if(xfrequency<127)
  {
  if(xfrequency<67 ) return  24;
  if(xfrequency<71 ) return  25;
  if(xfrequency<76 ) return  26;
  if(xfrequency<80 ) return  27;
  if(xfrequency<85 ) return  28;
  if(xfrequency<90 ) return  29;
  if(xfrequency<96 ) return  30;
  if(xfrequency<101) return  31;
  if(xfrequency<107) return  32;
  if(xfrequency<114) return  33;
  if(xfrequency<120) return  34;
                     return  35;
  }

  /* second octave */
  if(xfrequency<255)
  {
  if(xfrequency<136) return  36;
  if(xfrequency<143) return  37;
  if(xfrequency<152) return  38;
  if(xfrequency<160) return  39;
  if(xfrequency<170) return  40;
  if(xfrequency<180) return  41;
  if(xfrequency<190) return  42;
  if(xfrequency<202) return  43;
  if(xfrequency<215) return  44;
  if(xfrequency<227) return  45;
  if(xfrequency<240) return  46;
                     return  47;
  }

  /* third octave */
  if(xfrequency<512)
  {
   if(xfrequency<270)return  48;
   if(xfrequency<286)return  49;
   if(xfrequency<304)return  50;
   if(xfrequency<322)return  51;
   if(xfrequency<340)return  52;
   if(xfrequency<360)return  53;
   if(xfrequency<382)return  54;
   if(xfrequency<406)return  55;
   if(xfrequency<428)return  56;
   if(xfrequency<454)return  57;
   if(xfrequency<480)return  58;
                     return  59;
   }

  /* fourth octave */
  if(xfrequency<1024)
  {
  if(xfrequency<540)return 60; /* middle C (C4) */
  if(xfrequency<572)return 61;
  if(xfrequency<605)return 62;
  if(xfrequency<643)return 63;
  if(xfrequency<680)return 64;
  if(xfrequency<720)return 65;
  if(xfrequency<765)return 66;
  if(xfrequency<810)return 67;
  if(xfrequency<855)return 68;
  if(xfrequency<910)return 69;
  if(xfrequency<965)return 70;
                    return 71;
  }

  /* fifth octave */
  if(xfrequency<2048)
  {
  if(xfrequency<1080) return 72;
  if(xfrequency<1144) return 73;
  if(xfrequency<1210) return 74;
  if(xfrequency<1286) return 75;
  if(xfrequency<1340) return 76;
  if(xfrequency<1440) return 77;
  if(xfrequency<1530) return 78;
  if(xfrequency<1620) return 79;
  if(xfrequency<1710) return 80;
  if(xfrequency<1820) return 81;
  if(xfrequency<1930) return 82;
                      return 83;
  }

  if (xfrequency < 4096)
  {

   /* sixth octave */
   /* seventh, eight, and ninth octave ignored */
   if(xfrequency<2160)return 84;
   if(xfrequency<2288)return 85;
   if(xfrequency<2420)return 86;
   if(xfrequency<2572)return 87;
   if(xfrequency<2680)return 88;
   if(xfrequency<2880)return 89;
   if(xfrequency<3060)return 90;
   if(xfrequency<3240)return 91;
   if(xfrequency<3420)return 92;
   if(xfrequency<3640)return 93;
   if(xfrequency<3860)return 94;
                      return 95;
  }

   /* seventh octave */
   /* eight, and ninth octave ignored */
   if(xfrequency<4320)return 96;
   if(xfrequency<4576)return 97;
   if(xfrequency<4840)return 98;
   if(xfrequency<5144)return 99;
   if(xfrequency<5360)return 100;
   if(xfrequency<5760)return 101;
   if(xfrequency<6120)return 102;
   if(xfrequency<6480)return 103;
   if(xfrequency<6840)return 104;
   if(xfrequency<7280)return 105;
   if(xfrequency<7720)return 106;
   return 107;
}






void writeheader(FILE *fp2, char *fname, char *aname, int acnt)
{

   fprintf(fp2,
   "/* musical array created from IBM snd file %s */\n",fname);
   fprintf(fp2,
   "/* array structure is midi note number, duration (18.2 ticks sec) */\n");
   fprintf(fp2,
   "char %s%d[]={\n",aname,acnt);


}

main(int argc, char *argv[])
{
   FILE *fp,*fp2,*fp3;
   char *wordptr, c;
   unsigned freq, dur;
   int frequency,duration,counter=0, acnt=0;
   char scratchbuffer[128],writebuffer[128];

   if(argc!=2)
   {
    printf("SND filename required...");
    exit(0);
    }

   if((fp=fopen(argv[1],"rb"))==NULL)
   {
    perror(argv[1]);
    exit(0);
    }

   strcpy(scratchbuffer,argv[1]);
   wordptr=strtok(scratchbuffer,".");



   sprintf(writebuffer,"%s.S%02d",scratchbuffer,acnt);
   fp2=fopen(writebuffer,"w");
   writeheader(fp2, argv[1], scratchbuffer,acnt);
   acnt++;

   c=0;

    while((frequency=getw(fp))!=-1){
         duration=fgetc(fp);
         if (duration == 0) continue; /* ignore all notes of 0 duration */

         sound(frequency, duration);

         dur = duration;

         if((frequency=xplay(frequency))<0)
         {
            freq = 255; /* silence */
         }
         else
         {

		    freq = frequency;
         }
         fprintf(fp2,"%3d, %3d,",freq,dur);

         counter++;
         if(counter==8){         /* wide enough */
            fprintf(fp2,"\n");
            counter=0;
         }

         c = getch();
         if (c == 0)getch();

         if (c==27)break;

         if (c==13) {
				fprintf(fp2,"%3d, %3d};\n",0,0);
				fclose(fp2);
				sprintf(writebuffer,"%s.S%02d",scratchbuffer,acnt);
				fp2=fopen(writebuffer,"w");
				writeheader(fp2, argv[1], scratchbuffer,acnt);
				acnt++;
		 }
	}

  	 /* use 0 as the terminator */
  	if (c!=27 && c!=13) fprintf(fp2,"%3d, %3d};\n",0,0);

    fclose(fp);
    fclose(fp2);
    if(c==27 || c==13)remove(writebuffer);
    exit(0);

}